/**
 * Copyright 2006 (C) Bernd Bruegge, Timo Wolf - TU Muenchen
 * http://wwwbruegge.in.tum.de/Asteroids
 * 
 * This file is part of the Asteroids Teaching Design Pattern Project.
 * 
 * Asteroids is free software; you can redistribute it and/or modify it under the
 * terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * Asteroids is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
 * A PARTICULAR PURPOSE. See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * Asteroids; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA 02111-1307 USA
 */
package de.tum.in.wwwbruegge.asteroids.ui;

import java.awt.Canvas;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Image;
import java.awt.Point;
import java.awt.RenderingHints;
import java.awt.geom.AffineTransform;
import java.util.ArrayList;

import de.tum.in.wwwbruegge.asteroids.solidbodies.BigAsteroid;
import de.tum.in.wwwbruegge.asteroids.solidbodies.SolidBody;
import de.tum.in.wwwbruegge.asteroids.solidbodies.SpaceShuttle;

/**
 * @author Timo Wolf
 */
public class GameBoard extends Canvas {

	private static GameBoard INSTANCE;

	private static int NUMBER_OF_ASTEROIDS = 8;

	private static Dimension DEFAULT_SIZE = new Dimension(500, 300);

	private static Color backgroundColor = Color.black;

	private static Color starColor = Color.white;

	private ArrayList<SolidBody> solidBodies = new ArrayList<SolidBody>();

	private SpaceShuttle spaceShuttle;

	private Image backgroundImage;

	private Image spaceImage;

	private Graphics backgroundGraphics;

	private Dimension theSize;

	public static GameBoard getInstance() {
		if (INSTANCE == null) {
			INSTANCE = new GameBoard();
		}
		return INSTANCE;
	}

	private GameBoard() {
		setSize(getPreferredSize());
		Point position = new Point(getSize().width / 2, getSize().height / 2);
		this.spaceShuttle = new SpaceShuttle(position, 0);
	}

	public Dimension getPreferredSize() {
		return DEFAULT_SIZE;
	}

	public SolidBody[] getSolidBodies() {
		return solidBodies.toArray(new SolidBody[solidBodies.size()]);
	}

	public SpaceShuttle getSpaceShuttle() {
		return this.spaceShuttle;
	}

	public final synchronized void update(Graphics g) {
		if (theSize == null || theSize.width != getSize().width
				|| theSize.height != getSize().height) {
			theSize = getSize();
			backgroundImage = createImage(theSize.width, theSize.height);
			backgroundGraphics = backgroundImage.getGraphics();
		}
		paint(backgroundGraphics);
		g.drawImage(backgroundImage, 0, 0, null);
	}

	public void paint(Graphics g) {
		if (theSize == null || theSize.width != getSize().width
				|| theSize.height != getSize().height) {
			theSize = getSize();
			backgroundImage = createImage(theSize.width, theSize.height);
			backgroundGraphics = backgroundImage.getGraphics();
		}

		if (spaceImage == null || spaceImage.getWidth(this) != getWidth()
				|| spaceImage.getHeight(this) != getHeight()) {

			spaceImage = createImage(getWidth(), getHeight());
			Graphics spacegraphics = spaceImage.getGraphics();
			spacegraphics.setColor(backgroundColor);
			spacegraphics.fillRect(0, 0, getSize().width, getSize().height);
			spacegraphics.setColor(starColor);
			for (int i = 0; i < 50; i++) {
				int x = (int) (Math.random() * (double) getSize().width);
				int y = (int) (Math.random() * (double) getSize().height);
				spacegraphics.fillOval(x, y, 3, 3);
			}
		}
		Graphics2D g2d = (Graphics2D) backgroundGraphics;
		g2d.setRenderingHint(RenderingHints.KEY_ANTIALIASING,
				RenderingHints.VALUE_ANTIALIAS_ON);

		backgroundGraphics.drawImage(spaceImage, 0, 0, this);
		SolidBody[] solids = getSolidBodies();
		int num = solids.length;
		for (int i = 0; i < num; i++) {
			paintSolidBody(solids[i], backgroundGraphics);

		}
		g2d = (Graphics2D) g;
		g2d.setRenderingHint(RenderingHints.KEY_ANTIALIASING,
				RenderingHints.VALUE_ANTIALIAS_ON);
		g.drawImage(backgroundImage, 0, 0, this);
	}

	private void paintSolidBody(SolidBody solidbody, Graphics g) {
		Point position = solidbody.getPosition();
		Point canvasPosition = convertPosition(position);
		Image image = solidbody.getBody();
		Graphics2D g2d = (Graphics2D) g;
		AffineTransform origXform = g2d.getTransform();
		AffineTransform newXform = (AffineTransform) (origXform.clone());
		// center of rotation is center of the panel
		int xRot = canvasPosition.x + solidbody.getSize().width / 2;
		int yRot = canvasPosition.y + solidbody.getSize().height / 2;
		newXform.rotate(Math.toRadians(solidbody.getDirection()), xRot, yRot);
		g2d.setTransform(newXform);
		// draw image centered in panel
		g2d.drawImage(image, canvasPosition.x, canvasPosition.y, this);
		g2d.setTransform(origXform);
	}

	public void initiate() {
		this.solidBodies.clear();
		Point spaceshuttleposition = new Point(getSize().width / 2,
				getSize().height / 2);
		this.spaceShuttle.initiate(spaceshuttleposition, 0);

		this.solidBodies.add(spaceShuttle);
		for (int i = 0; i < NUMBER_OF_ASTEROIDS; i++) {
			Point position = new Point(
					(int) (Math.random() * (getSize().width - 30)), (int) (Math
							.random() * (getSize().height - 30)));

			SolidBody solidbody = new BigAsteroid(position, (int) (Math
					.random() * 360));
			addSolidBody(solidbody);
		}
		repaint();
	}

	public void addSolidBody(SolidBody solidbody) {
		solidBodies.add(solidbody);
		paintSolidBody(solidbody, getGraphics());
		repaint();
	}

	public void removeSolidBody(SolidBody solidbody) {
		solidBodies.remove(solidbody);
		repaint();
	}

	public Point convertPosition(Point toConvert) {
		return new Point(toConvert.x, getSize().height - toConvert.y);
	}
}
